package com.alihealth.d2d.provtest.service;

import com.alihealth.d2d.provtest.domain.DomesticProducer;
import com.alihealth.d2d.provtest.entity.VTTSBasic;
import org.springframework.stereotype.Service;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import javax.xml.XMLConstants;
import javax.xml.bind.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import javax.xml.transform.stream.StreamSource;
import java.io.StringReader;
import java.io.StringWriter;
import java.lang.reflect.Field;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author Jason
 * @date 2025年05月30日 11:41:26
 */
@Service
public class XmlParserService {

    private static final Map<Class<?>, JAXBContext> CONTEXT_CACHE = new ConcurrentHashMap<>();


    /**
     * 解析包含多个data节点的XML
     * @param xml 完整XML内容
     * @param dataClass 内部data节点对应的实体类
     */
    public <T> List<DomesticProducer> parseMultiDataXml(
            String xml,
            Class<T> dataClass) throws Exception {

        // 1. 参数校验
        if (xml == null || xml.trim().isEmpty()) {
            throw new IllegalArgumentException("XML内容不能为空");
        }

        // 2. 创建包含包装类和数据类的JAXB上下文
        JAXBContext context = JAXBContext.newInstance(
                VTTSBasic.class,
                dataClass  // 包含实际要解析的数据类型
        );
        // 3. 配置安全的Unmarshaller（防止XXE攻击）
        Unmarshaller unmarshaller = context.createUnmarshaller();
        XMLInputFactory xif = XMLInputFactory.newInstance();
        xif.setProperty(XMLInputFactory.IS_SUPPORTING_EXTERNAL_ENTITIES, false);
        xif.setProperty(XMLInputFactory.SUPPORT_DTD, false);

        // 4. 解析XML
        try (StringReader reader = new StringReader(xml)) {
//            XMLStreamReader xsr = xif.createXMLStreamReader(reader);
//            JAXBElement<VTTSBasic> root = unmarshaller.unmarshal(xsr, VTTSBasic.class);
//
//            // 5. 获取解析结果
//            VTTSBasic<T> wrapper = root.getValue();
//            List<T> result = wrapper.getDataset();
//
//            // 6. 类型安全检查
//            if (!result.isEmpty() && !dataClass.isInstance(result.get(0))) {
//                throw new ClassCastException("XML中包含类型不匹配的数据");
//            }
//
//            return result;
            // 创建 JAXBContext 时包含所有相关类
           // JAXBContext jc = JAXBContext.newInstance(VTTSBasic.class, Data.class);
            //Unmarshaller unmarshaller = jc.createUnmarshaller();

            // 解析 XML
            XMLStreamReader xsr = xif.createXMLStreamReader(reader);
            JAXBElement<VTTSBasic> root = unmarshaller.unmarshal(xsr, VTTSBasic.class);

            // 获取解析结果
            VTTSBasic wrapper = root.getValue();
            List<DomesticProducer> result = wrapper.getDataset();

            return result;
        }
    }


    @SuppressWarnings("unchecked")
    public static <T> VTTSBasic<T> parseXml(String xml, Class<T> dataType) throws Exception {
        JAXBContext context = getOrCreateContext(VTTSBasic.class, dataType);

        Unmarshaller unmarshaller = context.createUnmarshaller();
        JAXBElement<VTTSBasic> element = unmarshaller.unmarshal(
                new StreamSource(new StringReader(xml)),
                VTTSBasic.class
        );

        return element.getValue();
    }

    public static <T> String toXml(VTTSBasic<T> vttsBasic, Class<T> dataType) throws Exception {
        JAXBContext context = getOrCreateContext(VTTSBasic.class, dataType);

        Marshaller marshaller = context.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);

        StringWriter writer = new StringWriter();
        marshaller.marshal(vttsBasic, writer);

        return writer.toString();
    }

    private static JAXBContext getOrCreateContext(Class<?>... classes) throws JAXBException {
        String key = Arrays.stream(classes)
                .map(Class::getName)
                .sorted()
                .collect(Collectors.joining("|"));

        return CONTEXT_CACHE.computeIfAbsent(key.getClass(), k -> {
            try {
                // 为泛型类型创建JAXB上下文
                return JAXBContext.newInstance(classes);
            } catch (JAXBException e) {
                throw new RuntimeException("Failed to create JAXBContext", e);
            }
        });
    }
}
